/*
 *  exo02.c
 *  test pour un nombre premier
 */

#include <stdio.h>
#include <math.h>     /* 
                       * attention !
                       * pour compiler avec la librerie
                       * math.h, il faut utiliser l'option -lm
                       * (_l_ink library _m_ath) , _après_ le nom du
                       * source:
                       *  gcc exo02.c -lm
                       */

/* 
 * le comment ci-dessous est un forme pour decrire la fonction
 * c'est utile si on utilise beacoup de functions
 * on écrit aussi des suppositions sur les entrés, comme nombre positif
 * ou pointer non nul. @n est un convention pour denoter les variables
 * (pour le distinguer du reste du texte). On l'utilise comme une forme
 * de API (pour cuex qui ont programmé en java)
 */

/*
 * test de primalité
 * name: isPrime
 * @n :       un nombre entier
 * @return :  1 si @n est premier, 0 sinon
 */
short int isPrime(int n) { // short parce que on n'a pas besoin de plus
                           // short declare que le int occupe moins
                           // d'espace dans la mémoire (mais donc ses
                           // limites sont plus petits
  if(n <= 1) return 0;  // d'abord on se libère des cas trivials
  int i;
  const double rc = sqrt(n);  // on sauvegarde la racine carrè pour ne
                              // la calculer pas a chaque cycle
                              // const : cela declare que la variable
                              // ne sera pas changé. Pas obligatoire
                              // mais aide le compilateur à optimiser
                              // le programme
  for (i = 2; i <= rc; i++) {
    if (n%i == 0) return 0;
  }
  return 1;
}

/*
 * deuxième version, avec un peu moins d'operations :
 * cela teste la parité, après teste seulement sur les nombres impairs
 * noter comment on fait une iteration sur les impairs
 * 
 *  int isPrime(int n) {
 *    if (n <= 1 || n%2 == 0) return 0;
 *    int i;
 *    const double rc = sqrt(n);
 *    for (i = 3; i <= rc; i += 2) {
 *      if (n%i == 0) return 0;
 *    }
 *    return 1;
 *  }
 * 
 *  pour le curieux :
 *    http://en.wikipedia.org/wiki/Primality_test
 *  en fait les tests ci-dessu sont naïves
 *  un algorithme deterministe en temps polynomial est un découverte
 *  relativement recente.
 */

int main() {
  int x;
  printf("Bonjour !\n");
  printf("Svp, tapez un entier : ");
  scanf("%d",&x);
  if (isPrime(x)) {
    printf("Le nombre est premier !\n");
  } else {
    printf("Le nombre n'est pas premier...\n");
  }
}

/* autre possibilité : initialiser x à un nombre different de 0
 * et utiliser
 *    while(n != 0) {
 *      ...
 *    }
 *  mais cela utilise un condition en plus. En general, on
 *  utilise do { ... } while(cond) si on doit faire le cycle
 *  au moins une fois.
 */
